@@ -31,6 +31,7 @@ import threading
 
 from proxy_tools import proxy
 
+from dsl_parser import functions as dsl_functions
 from cloudify import context
 from cloudify._compat import queue
 from cloudify.manager import (get_bootstrap_context,
@@ -1256,6 +1257,9 @@ class CloudifyWorkflowContextInternal(object):
     def add_local_task(self, task):
         self.local_tasks_processor.add_task(task)
 
+    def evaluate_functions(self, deployment_id, ctx, payload):
+        return self.handler.evaluate_functions(deployment_id, ctx, payload)
+
 
 class LocalTasksProcessing(object):
 
@@ -1447,6 +1451,9 @@ class CloudifyWorkflowContextHandler(object):
     def start_execution(self, deployment_id, workflow_id, **kwargs):
         raise NotImplementedError('Implemented by subclasses')
 
+    def evaluate_functions(self, deployment_id, ctx, payload):
+        raise NotImplementedError('Implemented by subclasses')
+
 
 class RemoteContextHandler(CloudifyWorkflowContextHandler):
     def __init__(self, *args, **kwargs):
@@ -1669,6 +1676,11 @@ class RemoteContextHandler(CloudifyWorkflowContextHandler):
             **kwargs
         )
 
+    def evaluate_functions(self, deployment_id, ctx, payload):
+        result = \
+            self.rest_client.evaluate.functions(deployment_id, ctx, payload)
+        return result.get('payload')
+
 
 class RemoteCloudifyWorkflowContextHandler(RemoteContextHandler):
     _scaling_groups = None
@@ -1911,6 +1923,10 @@ class LocalCloudifyWorkflowContextHandler(CloudifyWorkflowContextHandler):
         raise RuntimeError('Starting executions from executions is not '
                            'implemented for local workflows')
 
+    def evaluate_functions(self, deployment_id, ctx, payload):
+        return dsl_functions.evaluate_functions(
+            payload=payload, context=ctx, storage=self.storage)
+
 
 class Modification(object):
 
@@ -1989,6 +2005,7 @@ class WorkflowDeploymentContext(context.DeploymentContext):
     def __init__(self, cloudify_context, workflow_ctx):
         super(WorkflowDeploymentContext, self).__init__(cloudify_context)
         self.workflow_ctx = workflow_ctx
+        self._resource_tags = None
 
     def start_modification(self, nodes):
         """Start deployment modification process
@@ -2016,6 +2033,22 @@ class WorkflowDeploymentContext(context.DeploymentContext):
     def scaling_groups(self):
         return self.workflow_ctx.internal.handler.scaling_groups
 
+    @property
+    def resource_tags(self):
+        """Resource tags associated wth this deployment."""
+        if self._resource_tags is None and self.workflow_ctx.internal:
+            raw_tags = self._context.get('deployment_resource_tags')
+            if raw_tags:
+                evaluated = self.workflow_ctx.internal.evaluate_functions(
+                    self.id,
+                    self._context,
+                    raw_tags,
+                )
+            else:
+                evaluated = {}
+            self._resource_tags = evaluated
+        return self._resource_tags
+
 
 def task_config(fn=None, **arguments):
     if fn is not None:
