@@ -1,18 +1,3 @@
-########
-# Copyright (c) 2014 GigaSpaces Technologies Ltd. All rights reserved
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-#    * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    * See the License for the specific language governing permissions and
-#    * limitations under the License.
-
 import warnings
 
 from cloudify_rest_client.responses import ListResponse
@@ -342,7 +327,8 @@ class DeploymentGroupsClient(object):
     def put(self, group_id, visibility=VisibilityState.TENANT,
             description=None, blueprint_id=None, default_inputs=None,
             labels=None, filter_id=None, deployment_ids=None,
-            new_deployments=None, deployments_from_group=None):
+            new_deployments=None, deployments_from_group=None,
+            creator=None, created_at=None):
         """Create or update the specified deployment group.
 
         Setting group deployments using this method (via either filter_id
@@ -366,21 +352,27 @@ class DeploymentGroupsClient(object):
             keys "id", "inputs", "labels"
         :param deployments_from_group: add all deployments belonging to the
             group given by this id
+        :param creator: Override the creator. Internal use only.
+        :param created_at: Override the creation timestamp. Internal use only.
         :return: the created deployment group
         """
+        data = {
+            'visibility': visibility,
+            'description': description,
+            'blueprint_id': blueprint_id,
+            'default_inputs': default_inputs,
+            'labels': labels,
+            'filter_id': filter_id,
+            'deployment_ids': deployment_ids,
+            'new_deployments': new_deployments,
+            'deployments_from_group': deployments_from_group,
+        }
+        if created_at:
+            data['created_at'] = created_at
+        if creator:
+            data['creator'] = creator
         response = self.api.put(
-            '/deployment-groups/{0}'.format(group_id),
-            data={
-                'visibility': visibility,
-                'description': description,
-                'blueprint_id': blueprint_id,
-                'default_inputs': default_inputs,
-                'labels': labels,
-                'filter_id': filter_id,
-                'deployment_ids': deployment_ids,
-                'new_deployments': new_deployments,
-                'deployments_from_group': deployments_from_group,
-            }
+            '/deployment-groups/{0}'.format(group_id), data=data,
         )
         return DeploymentGroup(response)
 
@@ -563,7 +555,8 @@ class DeploymentsClient(object):
     def get(self,
             deployment_id,
             _include=None,
-            all_sub_deployments=True
+            all_sub_deployments=True,
+            include_workdir=False,
             ):
         """
         Returns a deployment by its id.
@@ -581,7 +574,8 @@ class DeploymentsClient(object):
         response = self.api.get(
             uri,
             _include=_include,
-            params={'all_sub_deployments': all_sub_deployments}
+            params={'all_sub_deployments': all_sub_deployments,
+                    'include_workdir': include_workdir}
         )
         return Deployment(response)
 
@@ -594,7 +588,9 @@ class DeploymentsClient(object):
                site_name=None,
                runtime_only_evaluation=False,
                labels=None,
-               display_name=None):
+               display_name=None,
+               async_create=None,
+               _workdir_zip=None):
         """
         Creates a new deployment for the provided blueprint id and
         deployment id.
@@ -615,6 +611,9 @@ class DeploymentsClient(object):
         :param labels: The deployment's labels. A list of 1-entry
             dictionaries: [{<key1>: <value1>}, {<key2>: <value2>}, ...]'
         :param display_name: The deployment's display name.
+        :param async_create: if True, do not wait for the deployment
+            environment to finish creating
+        :param _workdir_zip: Internal only.
         :return: The created deployment.
         """
         assert blueprint_id
@@ -628,10 +627,17 @@ class DeploymentsClient(object):
             data['labels'] = labels
         if display_name:
             data['display_name'] = display_name
+        if _workdir_zip:
+            data['workdir_zip'] = _workdir_zip
         data['skip_plugins_validation'] = skip_plugins_validation
         data['runtime_only_evaluation'] = runtime_only_evaluation
         uri = '/deployments/{0}'.format(deployment_id)
-        response = self.api.put(uri, data, expected_status_code=201)
+        params = {}
+        if async_create is not None:
+            # if it's None, we just keep the server's default behaviour
+            params['async_create'] = async_create
+        response = self.api.put(
+            uri, data, params=params, expected_status_code=201)
         return Deployment(response)
 
     def delete(self, deployment_id,
@@ -693,7 +699,8 @@ class DeploymentsClient(object):
             data=data
         )
 
-    def update_labels(self, deployment_id, labels):
+    def update_labels(self, deployment_id, labels, creator=None,
+                      created_at=None):
         """
         Updates the deployment's labels.
 
@@ -703,6 +710,10 @@ class DeploymentsClient(object):
         :return: The deployment
         """
         data = {'labels': labels}
+        if creator:
+            data['creator'] = creator
+        if created_at:
+            data['created_at'] = created_at
         updated_dep = self.api.patch(
             '/deployments/{0}'.format(deployment_id), data=data)
         return Deployment(updated_dep)
